#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import with_statement

from contextlib import contextmanager
from subprocess import Popen, PIPE
import sys, os, errno, codecs, re, time, locale

RE_VERSION = re.compile('^version (?P<version>.*):$')

@contextmanager
def localectx(category, newlocale=None):
	old = locale.getlocale(category)
	locale.setlocale(category, newlocale)
	try:
		yield
	finally:
		locale.setlocale(category, old)

class CallError(Exception):
	pass

def call(*args):
	p = Popen(args,stdout=PIPE)
	s = p.stdout.read()
	code = p.wait()
	
	if code != 0:
		raise CallError('%s: exit code was: %d' % (' '.join(args), code), args, code)
	
	return s

def which(app):
	return call('which',app).rstrip()

def whatprovides(app,dowhich=True):
	if dowhich:
		app = which(app)
	
	pkg = call('rpm', '-q', '--whatprovides', app).rstrip()

	try:
		return pkg[:pkg.index('-')]
	except ValueError:
		return pkg

def main():
	if len(sys.argv) < 2:
		kde_version = 'kde4'
	else:
		kde_version = sys.argv[1].lower().strip()
	
	if len(sys.argv) < 3:
		release = 1
	else:
		release = int(sys.argv[2], 10)

	cleanup = []

	if 'kde3' == kde_version:
		services_path = 'usr/share/apps/konqueror/servicemenus'
		script        = 'get_yt_video.py'
		servicemenu   = 'get_yt_video.desktop'
		require       = set([
			whatprovides('kcookiejar'),
			whatprovides('kdialog'),
			whatprovides('python'),
			whatprovides('python-lxml', False),
			whatprovides('dcop')
		])
	elif 'kde4' == kde_version:
		try:
			services_path = call('kde4-config', '--install', 'services').rstrip()
		except OSError, e:
			if e.errno == errno.ENOENT:
				services_path = 'usr/share/kde4/services'
			else:
				raise

		script      = 'get_yt_video.py'
		servicemenu = 'get_yt_video-kde4.desktop'
		require     = set([
			whatprovides('kcookiejar4'),
			whatprovides('kdialog'),
			whatprovides('python'),
			whatprovides('python-lxml', False),
			whatprovides('qdbus')
		])

		cleanup.append(servicemenu)

		with codecs.open('get_yt_video.desktop','r',encoding='utf-8') as fin:
			with codecs.open(servicemenu,'w',encoding='utf-8') as fout:
				for line in fin:
					if line.startswith('ServiceTypes='):
						fout.write('''\
Type=Service
ServiceTypes=KonqPopupMenu/Plugin,text/html
''')
					else:
						fout.write(line)

	else:
		raise Exception('unknown KDE version: ' + kde_version)

	with codecs.open('Changelog','r',encoding='utf-8') as fin:
		version = RE_VERSION.match(fin.readline()).group('version').replace('-','')
		
		lines = []
		for line in fin:
			if RE_VERSION.match(line):
				break

			lines.append(line)
	
		changelog = ''.join(lines)

	langs = [lang for lang in os.listdir(u'l10n') if os.path.isdir(os.path.join(u'l10n',lang))]
	spec  = 'youtube-servicemenu-%s-%s-%d.spec' % (kde_version, version, release)
	pwd   = os.path.abspath(os.curdir)

	with codecs.open(spec,'w',encoding='utf-8') as fout:
		fout.write('''\
Summary: KDE service menu to download youtube videos.
Name: youtube-servicemenu-%(kde)s
Version: %(version)s
Release: %(release)d
Requires: %(require)s
Obsoletes: youtube-servicemenu < 1.6
License: GPL
Group: Applications/Multimedia
URL: http://www.kde-apps.org/content/show.php/Get+YouTube+Video+%%28improved%%29?content=41456
BuildRoot: /var/tmp/%%{name}-buildroot-%(pid)d
ExclusiveArch: noarch

%%description
With this konqueror service menu it is possible to download a youtube video
by rightclicking a link to the video and choosing "get youtube video" from the
actions menu.

%%install
mkdir -p "$RPM_BUILD_ROOT/usr/bin/"
install "%(pwd)s/%(script)s" "$RPM_BUILD_ROOT/usr/bin/%(script)s"
mkdir -p "$RPM_BUILD_ROOT/%(services_path)s"
install "%(pwd)s/%(servicemenu)s" "$RPM_BUILD_ROOT/%(services_path)s/get_yt_video.desktop"
mkdir -p "$RPM_BUILD_ROOT/usr/share/get_yt_video/"
''' % {
	'pwd': pwd,
	'pid': os.getpid(),
	'version': version,
	'kde': kde_version,
	'release': release,
	'require': ', '.join(require),
	'script': script,
	'services_path': services_path,
	'servicemenu': servicemenu,
})
		for lang in langs:
			fout.write('''\
mkdir -p "$RPM_BUILD_ROOT/usr/share/get_yt_video/%(lang)s/LC_MESSAGES"
install "%(pwd)s/l10n/%(lang)s/LC_MESSAGES/get_yt_video.mo" "$RPM_BUILD_ROOT/usr/share/get_yt_video/%(lang)s/LC_MESSAGES"
''' % {
	'pwd': pwd,
	'lang': lang,
})

		fout.write('''\
%%clean

%%files
/usr/bin/%(script)s
/%(services_path)s/get_yt_video.desktop
''' % {
	'script': script,
	'services_path': services_path,
})

		for lang in langs:
			fout.write('/usr/share/get_yt_video/%s/LC_MESSAGES/get_yt_video.mo\n' % lang)

		with localectx(locale.LC_TIME,('en','UTF8')):
			sdate = time.strftime('%a %b %d %Y')

		fout.write(u'''
%%changelog

* %s Mathias Panzenböck <grosser.meister.morti@gmx.net>
''' % sdate)

		fout.write(changelog)

	call('rpmbuild', '-bb', '--target', 'noarch', spec)

	for fname in cleanup:
		os.remove(fname)

if __name__ == '__main__':
	main()
